@@ -52,7 +52,7 @@ static const char bugs[] =
     "Report bugs to http://www.aleksey.com/xmlsec/bugs.html\n";
 
 static const char helpCommands1[] =     
-    "Usage: xmlsec <command> [<options>] [<file>]\n"
+    "Usage: xmlsec <command> [<options>] [<files>]\n"
     "\n"
     "xmlsec is a command line tool for signing, verifying, encrypting and\n"
     "decrypting XML documents. The allowed <command> values are:\n"
@@ -120,10 +120,18 @@ static const char helpListKeyData[] =
     "Usage: xmlsec list-key-data\n"
     "Prints the list of known key data klasses\n";
 
+static const char helpCheckKeyData[] =     
+    "Usage: xmlsec check-key-data <key-data-name> [<key-data-name> ... ]\n"
+    "Checks the given key-data against the list of known key-data klasses\n";
+
 static const char helpListTransforms[] =     
     "Usage: xmlsec list-transforms\n"
     "Prints the list of known transform klasses\n";
 
+static const char helpCheckTransforms[] =     
+    "Usage: xmlsec check-transforms <transform-name> [<transform-name> ... ]\n"
+    "Checks the given transforms against the list of known transform klasses\n";
+
 #define xmlSecAppCmdLineTopicGeneral		0x0001
 #define xmlSecAppCmdLineTopicDSigCommon		0x0002
 #define xmlSecAppCmdLineTopicDSigSign		0x0004
@@ -272,7 +280,7 @@ static xmlSecAppCmdLineParam pkcs8PemParam = {
     xmlSecAppCmdLineTopicKeysMngr,
     "--pkcs8-pem",
     "--privkey-p8-pem",
-    "--pkcs-pem[:<name>] <file>[,<cafile>[,<cafile>[...]]]"
+    "--pkcs8-pem[:<name>] <file>[,<cafile>[,<cafile>[...]]]"
     "\n\tload private key from PKCS8 PEM file and PEM certificates"
     "\n\tthat verify this key",
     xmlSecAppCmdLineParamTypeStringList,
@@ -907,7 +915,9 @@ typedef enum {
     xmlSecAppCommandUnknown = 0,
     xmlSecAppCommandHelp,
     xmlSecAppCommandListKeyData,
+    xmlSecAppCommandCheckKeyData,
     xmlSecAppCommandListTransforms,    
+    xmlSecAppCommandCheckTransforms,    
     xmlSecAppCommandVersion,
     xmlSecAppCommandKeys,
     xmlSecAppCommandSign,
@@ -971,7 +981,9 @@ static void			xmlSecAppPrintXkmsServerCtx	(xmlSecXkmsServerCtxPtr xkmsServerCtx)
 #endif /* XMLSEC_NO_XKMS */
 
 static void			xmlSecAppListKeyData		(void);
+static int			xmlSecAppCheckKeyData	    (const char * name);
 static void			xmlSecAppListTransforms		(void);
+static int			xmlSecAppCheckTransform	    (const char * name);
 
 static xmlSecTransformUriType	xmlSecAppGetUriType		(const char* string);
 static FILE* 			xmlSecAppOpenFile		(const char* filename);
@@ -1083,9 +1095,29 @@ int main(int argc, const char **argv) {
 	case xmlSecAppCommandListKeyData:
 	    xmlSecAppListKeyData();
 	    break;
+	case xmlSecAppCommandCheckKeyData:
+	    for(i = pos; i < argc; ++i) {
+            if(xmlSecAppCheckKeyData(argv[i]) < 0) {
+                fprintf(stderr, "Error: key data \"%s\" not found\n", argv[i]);
+                goto fail;
+            } else {
+                fprintf(stdout, "Key data \"%s\" found\n", argv[i]);
+            }
+	    }
+	    break;
 	case xmlSecAppCommandListTransforms:
 	    xmlSecAppListTransforms();
 	    break;	    
+	case xmlSecAppCommandCheckTransforms:
+	    for(i = pos; i < argc; ++i) {
+            if(xmlSecAppCheckTransform(argv[i]) < 0) {
+                fprintf(stderr, "Error: transform \"%s\" not found\n", argv[i]);
+                goto fail;
+            } else {
+                fprintf(stdout, "Transforms \"%s\" found\n", argv[i]);
+            }
+	    }
+	    break;	    
 	case xmlSecAppCommandKeys:
 	    for(i = pos; i < argc; ++i) {
     	    	if(xmlSecAppCryptoSimpleKeysMngrSave(gKeysMngr, argv[i], xmlSecKeyDataTypeAny) < 0) {
@@ -1988,12 +2020,28 @@ xmlSecAppListKeyData(void) {
     xmlSecKeyDataIdListDebugDump(xmlSecKeyDataIdsGet(), stdout);
 }
 
+static int 
+xmlSecAppCheckKeyData(const char * name) {
+    if(xmlSecKeyDataIdListFindByName(xmlSecKeyDataIdsGet(), BAD_CAST name, xmlSecKeyDataUsageAny) == xmlSecKeyDataIdUnknown) {
+        return -1;
+    }
+    return 0;
+}
+
 static void 
 xmlSecAppListTransforms(void) {
     fprintf(stdout, "Registered transform klasses:\n");
     xmlSecTransformIdListDebugDump(xmlSecTransformIdsGet(), stdout);
 }
 
+static int 
+xmlSecAppCheckTransform(const char * name) {
+    if(xmlSecTransformIdListFindByName(xmlSecTransformIdsGet(), BAD_CAST name, xmlSecTransformUsageAny) == xmlSecTransformIdUnknown) {
+        return -1;
+    }
+    return 0;
+}
+
 static int 
 xmlSecAppPrepareKeyInfoReadCtx(xmlSecKeyInfoCtxPtr keyInfoCtx) {
     xmlSecAppCmdLineValuePtr value;
@@ -2684,10 +2732,20 @@ xmlSecAppParseCommand(const char* cmd, xmlSecAppCmdLineParamTopic* cmdLineTopics
 	return(xmlSecAppCommandListKeyData);
     } else 
 
+    if((strcmp(cmd, "check-key-data") == 0) || (strcmp(cmd, "--check-key-data") == 0)) {
+	(*cmdLineTopics) = 0;
+	return(xmlSecAppCommandCheckKeyData);
+    } else 
+
     if((strcmp(cmd, "list-transforms") == 0) || (strcmp(cmd, "--list-transforms") == 0)) {
 	(*cmdLineTopics) = 0;
 	return(xmlSecAppCommandListTransforms);
     } else 
+
+    if((strcmp(cmd, "check-transforms") == 0) || (strcmp(cmd, "--check-transforms") == 0)) {
+	(*cmdLineTopics) = 0;
+	return(xmlSecAppCommandCheckTransforms);
+    } else 
     
     if((strcmp(cmd, "keys") == 0) || (strcmp(cmd, "--keys") == 0)) {
 	(*cmdLineTopics) = xmlSecAppCmdLineTopicGeneral | 
@@ -2787,9 +2845,15 @@ xmlSecAppPrintHelp(xmlSecAppCommand command, xmlSecAppCmdLineParamTopic topics)
     case xmlSecAppCommandListKeyData:
 	fprintf(stdout, "%s\n", helpListKeyData);
         break;
+    case xmlSecAppCommandCheckKeyData:
+	fprintf(stdout, "%s\n", helpCheckKeyData);
+        break;
     case xmlSecAppCommandListTransforms:
 	fprintf(stdout, "%s\n", helpListTransforms);
         break;
+    case xmlSecAppCommandCheckTransforms:
+	fprintf(stdout, "%s\n", helpCheckTransforms);
+        break;
     case xmlSecAppCommandKeys:
 	fprintf(stdout, "%s\n", helpKeys);
         break;
@@ -2853,7 +2917,7 @@ xmlSecAppOpenFile(const char* filename) {
     if((filename == NULL) || (strcmp(filename, "-") == 0)) {
 	return(stdout);
     }
-    file = fopen(filename, "w");
+    file = fopen(filename, "wb");
     if(file == NULL) {
 	fprintf(stderr, "Error: failed to open file \"%s\"\n", filename);
 	return(NULL);
