@@ -3,8 +3,8 @@
 cur_pwd=`pwd`
 today=`date +%F-%T`
 module=xmlsec
-branch=
-cvs_root=:pserver:aleksey@cvs.gnome.org:/cvs/gnome
+branch=trunk
+svn_uri=svn+ssh://aleksey@svn.gnome.org/svn/$module/$branch
 rpm_root=/usr/src/redhat
 build_root="$rpm_root/BUILD/xmlsec-build-area-$today"
 remote_root=aleksey@ftp.aleksey.com:/var/ftp/pub/xmlsec/releases
@@ -14,14 +14,10 @@ rm -rf "$build_root"
 mkdir -p "$build_root"
 cd "$build_root"
 
-if test "z$branch" != "z"; then 
-    echo "Cheking out module $module from branch $branch"
-    cvs -d $cvs_root -z3 co -P -r $branch $module > /dev/null
-else
-    echo "Cheking out module $module from tip"
-    cvs -d $cvs_root -z3 co -P $module > /dev/null
-fi
+echo "Checking out the module $module"
+svn checkout $svn_uri $module
 cd xmlsec
+find . -name ".svn" | xargs rm -r
 
 ./autogen.sh --prefix=/usr --sysconfdir=/etc
 make rpm-release
@@ -33,3 +29,4 @@ mv $tar_file $rpm_root/SOURCES
 echo "Cleanup"
 cd "$cur_pwd"
 rm -rf "$build_root"
+
